@@ -7,24 +7,19 @@ from itertools import zip_longest
 from math import ceil
 
 from lxml import etree
+from pypdf import PdfReader, PdfWriter
 from zeep.exceptions import Fault
 
-try:
-    from PyPDF2 import PdfReader, PdfWriter
-except ImportError:
-    from PyPDF2 import PdfFileReader as PdfReader, PdfFileWriter as PdfWriter
-
 from trytond.i18n import gettext
-from trytond.pool import Pool, PoolMeta
 from trytond.model import fields
 from trytond.model.exceptions import AccessError
-from trytond.wizard import Wizard, StateAction, StateTransition
-from trytond.transaction import Transaction
-
 from trytond.modules.stock_package_shipping.exceptions import (
     PackingValidationError)
+from trytond.pool import Pool, PoolMeta
+from trytond.transaction import Transaction
+from trytond.wizard import StateAction, StateTransition, Wizard
 
-from .configuration import get_client, SHIPMENT_SERVICE
+from .configuration import SHIPMENT_SERVICE, get_client
 from .exceptions import DPDError
 
 TRACKING_URL = 'https://tracking.dpd.de/status/%(code)s/parcel/%(reference)s'
@@ -75,6 +70,7 @@ class Package(metaclass=PoolMeta):
 
 
 class ShippingDPDMixin:
+    __slots__ = ()
 
     def validate_packing_dpd(self):
         warehouse = self.shipping_warehouse
@@ -126,7 +122,7 @@ class CreateDPDShipping(Wizard):
         Package = pool.get('stock.package')
 
         shipment = self.record
-        if shipment.reference:
+        if shipment.shipping_reference:
             raise AccessError(
                 gettext('stock_package_shipping_dpd'
                     '.msg_shipment_has_reference_number',
@@ -136,6 +132,7 @@ class CreateDPDShipping(Wizard):
         if not credential.depot or not credential.token:
             credential.update_token()
 
+        carrier = shipment.carrier
         shipping_client = get_client(credential.server, SHIPMENT_SERVICE)
         print_options = self.get_print_options(shipment)
         packages = shipment.root_packages
@@ -164,7 +161,10 @@ class CreateDPDShipping(Wizard):
                         count += 1
                         credential.update_token()
                         continue
-                raise
+                raise DPDError(gettext(
+                        'stock_package_shipping_dpd.'
+                        'msg_dpd_webservice_error',
+                        message=e.message)) from e
         else:
             raise DPDError(
                 gettext('stock_package_shipping_dpd.msg_dpd_login_error',
@@ -183,17 +183,16 @@ class CreateDPDShipping(Wizard):
         for page in iter_pdf_pages(reader):
             new_pdf = PdfWriter()
             new_label = BytesIO()
-            if hasattr(new_pdf, 'add_page'):
-                new_pdf.add_page(page)
-            else:
-                new_pdf.addPage(page)
+            new_pdf.add_page(page)
             new_pdf.write(new_label)
             labels.append(new_label)
 
-        shipment.reference = response.mpsId
+        shipment.shipping_reference = response.mpsId
         parcels = response.parcelInformation
         for package, label, parcel in zip_longest(packages, labels, parcels):
             package.shipping_label = fields.Binary.cast(label.getvalue())
+            package.shipping_label_mimetype = (
+                carrier.shipping_label_mimetype)
             package.shipping_reference = parcel.parcelLabelNumber
         Package.save(packages)
         shipment.save()
@@ -220,7 +219,7 @@ class CreateDPDShipping(Wizard):
             'paperFormat': shipment.carrier.dpd_paper_format,
             }
 
-    def shipping_party(self, party, address):
+    def shipping_party(self, party, address, usage=None):
         shipping_party = {
             'name1': address.party_full_name[:35],
             'name2': '',
@@ -232,16 +231,12 @@ class CreateDPDShipping(Wizard):
         if party.full_name != address.party_full_name:
             shipping_party['name2'] = party.full_name[:35]
 
-        phone = email = ''
-        for mechanism in party.contact_mechanisms:
-            if mechanism.type in {'phone', 'mobile'} and not phone:
-                phone = mechanism.value
-            if mechanism.type == 'email' and not email:
-                email = mechanism.value
-        if len(phone) <= 30:
-            shipping_party['phone'] = phone
-        if len(email) <= 50:
-            shipping_party['email'] = email
+        phone = address.contact_mechanism_get({'phone', 'mobile'}, usage=usage)
+        if phone and len(phone.value) <= 30:
+            shipping_party['phone'] = phone.value
+        email = address.contact_mechanism_get('email', usage=usage)
+        if email and len(email.value) <= 50:
+            shipping_party['email'] = email.value
 
         return shipping_party
 
@@ -290,5 +285,44 @@ class CreateDPDShipping(Wizard):
             'parcels': [self.get_parcel(p) for p in packages],
             'productAndServiceData': {
                 'orderType': 'consignment',
-                }
+                **self.get_notification(shipment),
+                },
             }
+
+    def get_notification(self, shipment, usage=None):
+        carrier = shipment.carrier
+        if not carrier.dpd_notification:
+            return {}
+        party = shipment.shipping_to
+        if party and party.lang:
+            lang_code = party.lang.code
+        else:
+            lang_code = Transaction().language
+        lang_code = lang_code.upper()
+        channel2type = {
+            'sms': {'mobile'},
+            }
+
+        channels = [
+            (1, 'email'),
+            (3, 'sms'),
+            ]
+        if carrier.dpd_notification == 'sms':
+            channels = reversed(channels)
+
+        for channel_id, channel in channels:
+            mechanism = party.contact_mechanism_get(
+                channel2type.get(channel, channel), usage=usage)
+            if not mechanism:
+                continue
+            value = mechanism.value
+            if len(value) > 50:
+                continue
+            return {
+                'predict': {
+                    'channel': channel_id,
+                    'value': value,
+                    'language': lang_code,
+                    },
+                }
+        return {}
